/* Driver template for the LEMON parser generator.
** The author disclaims copyright to this source code.
*/
/* First off, code is included that follows the "include" declaration
** in the input grammar file. */
#include <stdio.h>
#line 20 "parse.y"

#include <ason/ason.h>
#include <ason/read.h>
#include <ason/namespace.h>

#include <assert.h>
#include <stdlib.h>
#include <errno.h>
#include <stdarg.h>

#include "value.h"

typedef union {
	int64_t n;
	char *c;
	ason_t *value;
} token_t;

/**
 * Output data.
 **/
struct parse_data {
	ason_t *ret;
	ason_ns_t *ns;
	int failed;
};

/* Lemon has a problem with these */
#pragma GCC diagnostic ignored "-Wunused-parameter"
#pragma GCC diagnostic ignored "-Wunused-variable"

#line 40 "parse.c"
/* Next is all token values, in a form suitable for use by makeheaders.
** This section will be null unless lemon is run with the -m switch.
*/
/* 
** These constants (all generated automatically by the parser generator)
** specify the various kinds of tokens (terminals) that the parser
** understands. 
**
** Each symbol here is a terminal symbol in the grammar.
*/
/* Make sure the INTERFACE macro is defined.
*/
#ifndef INTERFACE
# define INTERFACE 1
#endif
/* The next thing included is series of defines which control
** various aspects of the generated parser.
**    YYCODETYPE         is the data type used for storing terminal
**                       and nonterminal numbers.  "unsigned char" is
**                       used if there are fewer than 250 terminals
**                       and nonterminals.  "int" is used otherwise.
**    YYNOCODE           is a number of type YYCODETYPE which corresponds
**                       to no legal terminal or nonterminal number.  This
**                       number is used to fill in empty slots of the hash 
**                       table.
**    YYFALLBACK         If defined, this indicates that one or more tokens
**                       have fall-back values which should be used if the
**                       original value of the token will not parse.
**    YYACTIONTYPE       is the data type used for storing terminal
**                       and nonterminal numbers.  "unsigned char" is
**                       used if there are fewer than 250 rules and
**                       states combined.  "int" is used otherwise.
**    asonLemonTOKENTYPE     is the data type used for minor tokens given 
**                       directly to the parser from the tokenizer.
**    YYMINORTYPE        is the data type used for all minor tokens.
**                       This is typically a union of many types, one of
**                       which is asonLemonTOKENTYPE.  The entry in the union
**                       for base tokens is called "yy0".
**    YYSTACKDEPTH       is the maximum depth of the parser's stack.  If
**                       zero the stack is dynamically sized using realloc()
**    asonLemonARG_SDECL     A static variable declaration for the %extra_argument
**    asonLemonARG_PDECL     A parameter declaration for the %extra_argument
**    asonLemonARG_STORE     Code to store %extra_argument into yypParser
**    asonLemonARG_FETCH     Code to extract %extra_argument from yypParser
**    YYNSTATE           the combined number of states.
**    YYNRULE            the number of rules in the grammar
**    YYERRORSYMBOL      is the code number of the error symbol.  If not
**                       defined, then do no error processing.
*/
#define YYCODETYPE unsigned char
#define YYNOCODE 40
#define YYACTIONTYPE unsigned char
#define asonLemonTOKENTYPE token_t
typedef union {
  int yyinit;
  asonLemonTOKENTYPE yy0;
  ason_t * yy57;
} YYMINORTYPE;
#ifndef YYSTACKDEPTH
#define YYSTACKDEPTH 100
#endif
#define asonLemonARG_SDECL struct parse_data *data;
#define asonLemonARG_PDECL ,struct parse_data *data
#define asonLemonARG_FETCH struct parse_data *data = yypParser->data
#define asonLemonARG_STORE yypParser->data = data
#define YYNSTATE 58
#define YYNRULE 37
#define YY_NO_ACTION      (YYNSTATE+YYNRULE+2)
#define YY_ACCEPT_ACTION  (YYNSTATE+YYNRULE+1)
#define YY_ERROR_ACTION   (YYNSTATE+YYNRULE)

/* The yyzerominor constant is used to initialize instances of
** YYMINORTYPE objects to zero. */
static const YYMINORTYPE yyzerominor = { 0 };

/* Define the yytestcase() macro to be a no-op if is not already defined
** otherwise.
**
** Applications can choose to define yytestcase() in the %include section
** to a macro that can assist in verifying code coverage.  For production
** code the yytestcase() macro should be turned off.  But it is useful
** for testing.
*/
#ifndef yytestcase
# define yytestcase(X)
#endif


/* Next are the tables used to determine what action to take based on the
** current state and lookahead token.  These tables are used to implement
** functions that take a state number and lookahead value and return an
** action integer.  
**
** Suppose the action integer is N.  Then the action is determined as
** follows
**
**   0 <= N < YYNSTATE                  Shift N.  That is, push the lookahead
**                                      token onto the stack and goto state N.
**
**   YYNSTATE <= N < YYNSTATE+YYNRULE   Reduce by rule N-YYNSTATE.
**
**   N == YYNSTATE+YYNRULE              A syntax error has occurred.
**
**   N == YYNSTATE+YYNRULE+1            The parser accepts its input.
**
**   N == YYNSTATE+YYNRULE+2            No such action.  Denotes unused
**                                      slots in the yy_action[] table.
**
** The action table is constructed as a single large table named yy_action[].
** Given state S and lookahead X, the action is computed as
**
**      yy_action[ yy_shift_ofst[S] + X ]
**
** If the index value yy_shift_ofst[S]+X is out of range or if the value
** yy_lookahead[yy_shift_ofst[S]+X] is not equal to X or if yy_shift_ofst[S]
** is equal to YY_SHIFT_USE_DFLT, it means that the action is not in the table
** and that yy_default[S] should be used instead.  
**
** The formula above is for computing the action when the lookahead is
** a terminal symbol.  If the lookahead is a non-terminal (as occurs after
** a reduce action) then the yy_reduce_ofst[] array is used in place of
** the yy_shift_ofst[] array and YY_REDUCE_USE_DFLT is used in place of
** YY_SHIFT_USE_DFLT.
**
** The following are the tables generated in this section:
**
**  yy_action[]        A single table containing all actions.
**  yy_lookahead[]     A table containing the lookahead for each entry in
**                     yy_action.  Used to detect hash collisions.
**  yy_shift_ofst[]    For each state, the offset into yy_action for
**                     shifting terminals.
**  yy_reduce_ofst[]   For each state, the offset into yy_action for
**                     shifting non-terminals after a reduce.
**  yy_default[]       Default action for each state.
*/
#define YY_ACTTAB_COUNT (150)
static const YYACTIONTYPE yy_action[] = {
 /*     0 */    39,   58,    9,   53,   52,   51,   50,   49,   48,    2,
 /*    10 */    34,   46,   45,   12,    8,   41,    5,   39,    1,    9,
 /*    20 */    53,   52,   51,   50,   49,   48,    2,   25,   46,   45,
 /*    30 */    12,   54,   41,    5,   31,   40,    9,   53,   52,   51,
 /*    40 */    50,   49,   48,    2,   59,   46,   45,   12,   39,   41,
 /*    50 */     5,   53,   52,   51,   50,   49,   48,    2,    4,   46,
 /*    60 */    45,   12,   25,   41,    5,    8,   56,   24,   18,   47,
 /*    70 */    29,   17,   96,   32,   25,   15,   36,   13,   56,   24,
 /*    80 */    18,   47,   29,   17,   25,   30,    7,   11,   56,   24,
 /*    90 */    18,   47,   43,   28,   25,   35,   42,   20,   56,   24,
 /*   100 */    16,   47,   25,   19,   44,   22,   56,   24,   16,   47,
 /*   110 */    25,   23,    6,   10,   56,   24,   27,   47,   25,   22,
 /*   120 */    37,   33,   56,   24,   21,   47,   25,   38,   97,   97,
 /*   130 */    56,   24,   14,   47,   25,   97,   25,   97,   56,   26,
 /*   140 */    56,   47,   25,   57,    8,   97,   56,    3,   97,   55,
};
static const YYCODETYPE yy_lookahead[] = {
 /*     0 */     8,    0,   10,   11,   12,   13,   14,   15,   16,   17,
 /*    10 */    18,   19,   20,   21,    4,   23,   24,    8,    1,   10,
 /*    20 */    11,   12,   13,   14,   15,   16,   17,   27,   19,   20,
 /*    30 */    21,   31,   23,   24,    8,   25,   10,   11,   12,   13,
 /*    40 */    14,   15,   16,   17,    0,   19,   20,   21,    8,   23,
 /*    50 */    24,   11,   12,   13,   14,   15,   16,   17,    2,   19,
 /*    60 */    20,   21,   27,   23,   24,    4,   31,   32,   33,   34,
 /*    70 */    35,   36,   37,   38,   27,   29,   30,    7,   31,   32,
 /*    80 */    33,   34,   35,   36,   27,   38,    9,    6,   31,   32,
 /*    90 */    33,   34,   22,   36,   27,   28,   22,   15,   31,   32,
 /*   100 */    33,   34,   27,   28,   22,   23,   31,   32,   33,   34,
 /*   110 */    27,   15,    6,    5,   31,   32,   33,   34,   27,   23,
 /*   120 */    22,   18,   31,   32,   33,   34,   27,   30,   39,   39,
 /*   130 */    31,   32,   33,   34,   27,   39,   27,   39,   31,   32,
 /*   140 */    31,   34,   27,   34,    4,   39,   31,    7,   39,   34,
};
#define YY_SHIFT_USE_DFLT (-9)
#define YY_SHIFT_COUNT (32)
#define YY_SHIFT_MIN   (-8)
#define YY_SHIFT_MAX   (140)
static const short yy_shift_ofst[] = {
 /*     0 */    26,    9,   -8,    9,    9,    9,    9,    9,    9,    9,
 /*    10 */     9,   40,   82,   96,   10,   70,  140,   77,   61,  103,
 /*    20 */    98,   61,  106,   74,  108,   81,  108,   61,   77,   56,
 /*    30 */    44,   17,    1,
};
#define YY_REDUCE_USE_DFLT (-1)
#define YY_REDUCE_COUNT (13)
#define YY_REDUCE_MIN   (0)
#define YY_REDUCE_MAX   (115)
static const signed char yy_reduce_ofst[] = {
 /*     0 */    35,   47,   75,   67,   57,   99,   91,   83,  107,  115,
 /*    10 */   109,    0,   46,   97,
};
static const YYACTIONTYPE yy_default[] = {
 /*     0 */    95,   95,   95,   95,   95,   95,   95,   95,   95,   95,
 /*    10 */    95,   95,   95,   95,   95,   95,   90,   61,   63,   95,
 /*    20 */    95,   92,   95,   95,   65,   71,   66,   64,   62,   60,
 /*    30 */    95,   89,   95,   80,   79,   91,   93,   86,   94,   89,
 /*    40 */    88,   87,   85,   84,   83,   82,   81,   67,   78,   77,
 /*    50 */    76,   75,   74,   73,   72,   70,   69,   68,
};

/* The next table maps tokens into fallback tokens.  If a construct
** like the following:
** 
**      %fallback ID X Y Z.
**
** appears in the grammar, then ID becomes a fallback token for X, Y,
** and Z.  Whenever one of the tokens X, Y, or Z is input to the parser
** but it does not parse, the type of the token is changed to ID and
** the parse is retried before an error is thrown.
*/
#ifdef YYFALLBACK
static const YYCODETYPE yyFallback[] = {
};
#endif /* YYFALLBACK */

/* The following structure represents a single element of the
** parser's stack.  Information stored includes:
**
**   +  The state number for the parser at this level of the stack.
**
**   +  The value of the token stored at this level of the stack.
**      (In other words, the "major" token.)
**
**   +  The semantic value stored at this level of the stack.  This is
**      the information used by the action routines in the grammar.
**      It is sometimes called the "minor" token.
*/
struct yyStackEntry {
  YYACTIONTYPE stateno;  /* The state-number */
  YYCODETYPE major;      /* The major token value.  This is the code
                         ** number for the token at this stack level */
  YYMINORTYPE minor;     /* The user-supplied minor token value.  This
                         ** is the value of the token  */
};
typedef struct yyStackEntry yyStackEntry;

/* The state of the parser is completely contained in an instance of
** the following structure */
struct yyParser {
  int yyidx;                    /* Index of top element in stack */
#ifdef YYTRACKMAXSTACKDEPTH
  int yyidxMax;                 /* Maximum value of yyidx */
#endif
  int yyerrcnt;                 /* Shifts left before out of the error */
  asonLemonARG_SDECL                /* A place to hold %extra_argument */
#if YYSTACKDEPTH<=0
  int yystksz;                  /* Current side of the stack */
  yyStackEntry *yystack;        /* The parser's stack */
#else
  yyStackEntry yystack[YYSTACKDEPTH];  /* The parser's stack */
#endif
};
typedef struct yyParser yyParser;

#ifndef NDEBUG
#include <stdio.h>
static FILE *yyTraceFILE = 0;
static char *yyTracePrompt = 0;
#endif /* NDEBUG */

#ifndef NDEBUG
/* 
** Turn parser tracing on by giving a stream to which to write the trace
** and a prompt to preface each trace message.  Tracing is turned off
** by making either argument NULL 
**
** Inputs:
** <ul>
** <li> A FILE* to which trace output should be written.
**      If NULL, then tracing is turned off.
** <li> A prefix string written at the beginning of every
**      line of trace output.  If NULL, then tracing is
**      turned off.
** </ul>
**
** Outputs:
** None.
*/
void asonLemonTrace(FILE *TraceFILE, char *zTracePrompt){
  yyTraceFILE = TraceFILE;
  yyTracePrompt = zTracePrompt;
  if( yyTraceFILE==0 ) yyTracePrompt = 0;
  else if( yyTracePrompt==0 ) yyTraceFILE = 0;
}
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing shifts, the names of all terminals and nonterminals
** are required.  The following table supplies these names */
static const char *const yyTokenName[] = { 
  "$",             "ASSIGN",        "EQUAL",         "SUBSET",      
  "UNION",         "INTERSECT",     "COLON",         "COMMA",       
  "SYMBOL",        "REPR",          "NOT",           "PREBAKED",    
  "EMPTY",         "NULL",          "UNIVERSE",      "WILD",        
  "NUMBER",        "START_LIST",    "END_LIST",      "TRUE",        
  "FALSE",         "START_OBJ",     "END_OBJ",       "STRING",      
  "O_PAREN",       "C_PAREN",       "error",         "value",       
  "list",          "kv_list",       "kv_pair",       "join",        
  "intersect",     "union",         "comp",          "equality",    
  "repr",          "assignment",    "result",      
};
#endif /* NDEBUG */

#ifndef NDEBUG
/* For tracing reduce actions, the names of all rules are required.
*/
static const char *const yyRuleName[] = {
 /*   0 */ "assignment ::= result",
 /*   1 */ "assignment ::= SYMBOL ASSIGN result",
 /*   2 */ "result ::= equality",
 /*   3 */ "equality ::= repr",
 /*   4 */ "equality ::= equality EQUAL repr",
 /*   5 */ "repr ::= union",
 /*   6 */ "repr ::= repr REPR union",
 /*   7 */ "union ::= intersect",
 /*   8 */ "union ::= union UNION intersect",
 /*   9 */ "intersect ::= comp",
 /*  10 */ "intersect ::= intersect INTERSECT comp",
 /*  11 */ "comp ::= join",
 /*  12 */ "comp ::= NOT comp",
 /*  13 */ "join ::= value",
 /*  14 */ "join ::= value COLON join",
 /*  15 */ "value ::= PREBAKED",
 /*  16 */ "value ::= EMPTY",
 /*  17 */ "value ::= NULL",
 /*  18 */ "value ::= UNIVERSE",
 /*  19 */ "value ::= WILD",
 /*  20 */ "value ::= NUMBER",
 /*  21 */ "value ::= START_LIST END_LIST",
 /*  22 */ "value ::= START_LIST list END_LIST",
 /*  23 */ "value ::= TRUE",
 /*  24 */ "value ::= FALSE",
 /*  25 */ "value ::= START_OBJ END_OBJ",
 /*  26 */ "value ::= START_OBJ kv_list END_OBJ",
 /*  27 */ "value ::= START_OBJ kv_list COMMA WILD END_OBJ",
 /*  28 */ "value ::= START_OBJ WILD END_OBJ",
 /*  29 */ "value ::= STRING",
 /*  30 */ "value ::= O_PAREN union C_PAREN",
 /*  31 */ "value ::= SYMBOL",
 /*  32 */ "list ::= union",
 /*  33 */ "list ::= union COMMA list",
 /*  34 */ "kv_pair ::= STRING COLON union",
 /*  35 */ "kv_list ::= kv_pair",
 /*  36 */ "kv_list ::= kv_list COMMA kv_pair",
};
#endif /* NDEBUG */


#if YYSTACKDEPTH<=0
/*
** Try to increase the size of the parser stack.
*/
static void yyGrowStack(yyParser *p){
  int newSize;
  yyStackEntry *pNew;

  newSize = p->yystksz*2 + 100;
  pNew = realloc(p->yystack, newSize*sizeof(pNew[0]));
  if( pNew ){
    p->yystack = pNew;
    p->yystksz = newSize;
#ifndef NDEBUG
    if( yyTraceFILE ){
      fprintf(yyTraceFILE,"%sStack grows to %d entries!\n",
              yyTracePrompt, p->yystksz);
    }
#endif
  }
}
#endif

/* 
** This function allocates a new parser.
** The only argument is a pointer to a function which works like
** malloc.
**
** Inputs:
** A pointer to the function used to allocate memory.
**
** Outputs:
** A pointer to a parser.  This pointer is used in subsequent calls
** to asonLemon and asonLemonFree.
*/
void *asonLemonAlloc(void *(*mallocProc)(size_t)){
  yyParser *pParser;
  pParser = (yyParser*)(*mallocProc)( (size_t)sizeof(yyParser) );
  if( pParser ){
    pParser->yyidx = -1;
#ifdef YYTRACKMAXSTACKDEPTH
    pParser->yyidxMax = 0;
#endif
#if YYSTACKDEPTH<=0
    pParser->yystack = NULL;
    pParser->yystksz = 0;
    yyGrowStack(pParser);
#endif
  }
  return pParser;
}

/* The following function deletes the value associated with a
** symbol.  The symbol can be either a terminal or nonterminal.
** "yymajor" is the symbol code, and "yypminor" is a pointer to
** the value.
*/
static void yy_destructor(
  yyParser *yypParser,    /* The parser */
  YYCODETYPE yymajor,     /* Type code for object to destroy */
  YYMINORTYPE *yypminor   /* The object to be destroyed */
){
  asonLemonARG_FETCH;
  switch( yymajor ){
    /* Here is inserted the actions which take place when a
    ** terminal or non-terminal is destroyed.  This can happen
    ** when the symbol is popped from the stack during a
    ** reduce or during error processing or when a parser is 
    ** being destroyed before it is finished parsing.
    **
    ** Note: during a reduce, the only symbols destroyed are those
    ** which appear on the RHS of the rule, but which are not used
    ** inside the C code.
    */
    case 27: /* value */
    case 28: /* list */
    case 29: /* kv_list */
    case 30: /* kv_pair */
    case 31: /* join */
    case 32: /* intersect */
    case 33: /* union */
    case 34: /* comp */
    case 35: /* equality */
    case 36: /* repr */
{
#line 79 "parse.y"
 ason_destroy((yypminor->yy57)); 
#line 473 "parse.c"
}
      break;
    default:  break;   /* If no destructor action specified: do nothing */
  }
}

/*
** Pop the parser's stack once.
**
** If there is a destructor routine associated with the token which
** is popped from the stack, then call it.
**
** Return the major token number for the symbol popped.
*/
static int yy_pop_parser_stack(yyParser *pParser){
  YYCODETYPE yymajor;
  yyStackEntry *yytos = &pParser->yystack[pParser->yyidx];

  if( pParser->yyidx<0 ) return 0;
#ifndef NDEBUG
  if( yyTraceFILE && pParser->yyidx>=0 ){
    fprintf(yyTraceFILE,"%sPopping %s\n",
      yyTracePrompt,
      yyTokenName[yytos->major]);
  }
#endif
  yymajor = yytos->major;
  yy_destructor(pParser, yymajor, &yytos->minor);
  pParser->yyidx--;
  return yymajor;
}

/* 
** Deallocate and destroy a parser.  Destructors are all called for
** all stack elements before shutting the parser down.
**
** Inputs:
** <ul>
** <li>  A pointer to the parser.  This should be a pointer
**       obtained from asonLemonAlloc.
** <li>  A pointer to a function used to reclaim memory obtained
**       from malloc.
** </ul>
*/
void asonLemonFree(
  void *p,                    /* The parser to be deleted */
  void (*freeProc)(void*)     /* Function used to reclaim memory */
){
  yyParser *pParser = (yyParser*)p;
  if( pParser==0 ) return;
  while( pParser->yyidx>=0 ) yy_pop_parser_stack(pParser);
#if YYSTACKDEPTH<=0
  free(pParser->yystack);
#endif
  (*freeProc)((void*)pParser);
}

/*
** Return the peak depth of the stack for a parser.
*/
#ifdef YYTRACKMAXSTACKDEPTH
int asonLemonStackPeak(void *p){
  yyParser *pParser = (yyParser*)p;
  return pParser->yyidxMax;
}
#endif

/*
** Find the appropriate action for a parser given the terminal
** look-ahead token iLookAhead.
**
** If the look-ahead token is YYNOCODE, then check to see if the action is
** independent of the look-ahead.  If it is, return the action, otherwise
** return YY_NO_ACTION.
*/
static int yy_find_shift_action(
  yyParser *pParser,        /* The parser */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
  int stateno = pParser->yystack[pParser->yyidx].stateno;
 
  if( stateno>YY_SHIFT_COUNT
   || (i = yy_shift_ofst[stateno])==YY_SHIFT_USE_DFLT ){
    return yy_default[stateno];
  }
  assert( iLookAhead!=YYNOCODE );
  i += iLookAhead;
  if( i<0 || i>=YY_ACTTAB_COUNT || yy_lookahead[i]!=iLookAhead ){
    if( iLookAhead>0 ){
#ifdef YYFALLBACK
      YYCODETYPE iFallback;            /* Fallback token */
      if( iLookAhead<sizeof(yyFallback)/sizeof(yyFallback[0])
             && (iFallback = yyFallback[iLookAhead])!=0 ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE, "%sFALLBACK %s => %s\n",
             yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[iFallback]);
        }
#endif
        return yy_find_shift_action(pParser, iFallback);
      }
#endif
#ifdef YYWILDCARD
      {
        int j = i - iLookAhead + YYWILDCARD;
        if( 
#if YY_SHIFT_MIN+YYWILDCARD<0
          j>=0 &&
#endif
#if YY_SHIFT_MAX+YYWILDCARD>=YY_ACTTAB_COUNT
          j<YY_ACTTAB_COUNT &&
#endif
          yy_lookahead[j]==YYWILDCARD
        ){
#ifndef NDEBUG
          if( yyTraceFILE ){
            fprintf(yyTraceFILE, "%sWILDCARD %s => %s\n",
               yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[YYWILDCARD]);
          }
#endif /* NDEBUG */
          return yy_action[j];
        }
      }
#endif /* YYWILDCARD */
    }
    return yy_default[stateno];
  }else{
    return yy_action[i];
  }
}

/*
** Find the appropriate action for a parser given the non-terminal
** look-ahead token iLookAhead.
**
** If the look-ahead token is YYNOCODE, then check to see if the action is
** independent of the look-ahead.  If it is, return the action, otherwise
** return YY_NO_ACTION.
*/
static int yy_find_reduce_action(
  int stateno,              /* Current state number */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
#ifdef YYERRORSYMBOL
  if( stateno>YY_REDUCE_COUNT ){
    return yy_default[stateno];
  }
#else
  assert( stateno<=YY_REDUCE_COUNT );
#endif
  i = yy_reduce_ofst[stateno];
  assert( i!=YY_REDUCE_USE_DFLT );
  assert( iLookAhead!=YYNOCODE );
  i += iLookAhead;
#ifdef YYERRORSYMBOL
  if( i<0 || i>=YY_ACTTAB_COUNT || yy_lookahead[i]!=iLookAhead ){
    return yy_default[stateno];
  }
#else
  assert( i>=0 && i<YY_ACTTAB_COUNT );
  assert( yy_lookahead[i]==iLookAhead );
#endif
  return yy_action[i];
}

/*
** The following routine is called if the stack overflows.
*/
static void yyStackOverflow(yyParser *yypParser, YYMINORTYPE *yypMinor){
   asonLemonARG_FETCH;
   yypParser->yyidx--;
#ifndef NDEBUG
   if( yyTraceFILE ){
     fprintf(yyTraceFILE,"%sStack Overflow!\n",yyTracePrompt);
   }
#endif
   while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will execute if the parser
   ** stack every overflows */
   asonLemonARG_STORE; /* Suppress warning about unused %extra_argument var */
}

/*
** Perform a shift action.
*/
static void yy_shift(
  yyParser *yypParser,          /* The parser to be shifted */
  int yyNewState,               /* The new state to shift in */
  int yyMajor,                  /* The major token to shift in */
  YYMINORTYPE *yypMinor         /* Pointer to the minor token to shift in */
){
  yyStackEntry *yytos;
  yypParser->yyidx++;
#ifdef YYTRACKMAXSTACKDEPTH
  if( yypParser->yyidx>yypParser->yyidxMax ){
    yypParser->yyidxMax = yypParser->yyidx;
  }
#endif
#if YYSTACKDEPTH>0 
  if( yypParser->yyidx>=YYSTACKDEPTH ){
    yyStackOverflow(yypParser, yypMinor);
    return;
  }
#else
  if( yypParser->yyidx>=yypParser->yystksz ){
    yyGrowStack(yypParser);
    if( yypParser->yyidx>=yypParser->yystksz ){
      yyStackOverflow(yypParser, yypMinor);
      return;
    }
  }
#endif
  yytos = &yypParser->yystack[yypParser->yyidx];
  yytos->stateno = (YYACTIONTYPE)yyNewState;
  yytos->major = (YYCODETYPE)yyMajor;
  yytos->minor = *yypMinor;
#ifndef NDEBUG
  if( yyTraceFILE && yypParser->yyidx>0 ){
    int i;
    fprintf(yyTraceFILE,"%sShift %d\n",yyTracePrompt,yyNewState);
    fprintf(yyTraceFILE,"%sStack:",yyTracePrompt);
    for(i=1; i<=yypParser->yyidx; i++)
      fprintf(yyTraceFILE," %s",yyTokenName[yypParser->yystack[i].major]);
    fprintf(yyTraceFILE,"\n");
  }
#endif
}

/* The following table contains information about every rule that
** is used during the reduce.
*/
static const struct {
  YYCODETYPE lhs;         /* Symbol on the left-hand side of the rule */
  unsigned char nrhs;     /* Number of right-hand side symbols in the rule */
} yyRuleInfo[] = {
  { 37, 1 },
  { 37, 3 },
  { 38, 1 },
  { 35, 1 },
  { 35, 3 },
  { 36, 1 },
  { 36, 3 },
  { 33, 1 },
  { 33, 3 },
  { 32, 1 },
  { 32, 3 },
  { 34, 1 },
  { 34, 2 },
  { 31, 1 },
  { 31, 3 },
  { 27, 1 },
  { 27, 1 },
  { 27, 1 },
  { 27, 1 },
  { 27, 1 },
  { 27, 1 },
  { 27, 2 },
  { 27, 3 },
  { 27, 1 },
  { 27, 1 },
  { 27, 2 },
  { 27, 3 },
  { 27, 5 },
  { 27, 3 },
  { 27, 1 },
  { 27, 3 },
  { 27, 1 },
  { 28, 1 },
  { 28, 3 },
  { 30, 3 },
  { 29, 1 },
  { 29, 3 },
};

static void yy_accept(yyParser*);  /* Forward Declaration */

/*
** Perform a reduce action and the shift that must immediately
** follow the reduce.
*/
static void yy_reduce(
  yyParser *yypParser,         /* The parser */
  int yyruleno                 /* Number of the rule by which to reduce */
){
  int yygoto;                     /* The next state */
  int yyact;                      /* The next action */
  YYMINORTYPE yygotominor;        /* The LHS of the rule reduced */
  yyStackEntry *yymsp;            /* The top of the parser's stack */
  int yysize;                     /* Amount to pop the stack */
  asonLemonARG_FETCH;
  yymsp = &yypParser->yystack[yypParser->yyidx];
#ifndef NDEBUG
  if( yyTraceFILE && yyruleno>=0 
        && yyruleno<(int)(sizeof(yyRuleName)/sizeof(yyRuleName[0])) ){
    fprintf(yyTraceFILE, "%sReduce [%s].\n", yyTracePrompt,
      yyRuleName[yyruleno]);
  }
#endif /* NDEBUG */

  /* Silence complaints from purify about yygotominor being uninitialized
  ** in some cases when it is copied into the stack after the following
  ** switch.  yygotominor is uninitialized when a rule reduces that does
  ** not set the value of its left-hand side nonterminal.  Leaving the
  ** value of the nonterminal uninitialized is utterly harmless as long
  ** as the value is never used.  So really the only thing this code
  ** accomplishes is to quieten purify.  
  **
  ** 2007-01-16:  The wireshark project (www.wireshark.org) reports that
  ** without this code, their parser segfaults.  I'm not sure what there
  ** parser is doing to make this happen.  This is the second bug report
  ** from wireshark this week.  Clearly they are stressing Lemon in ways
  ** that it has not been previously stressed...  (SQLite ticket #2172)
  */
  /*memset(&yygotominor, 0, sizeof(yygotominor));*/
  yygotominor = yyzerominor;


  switch( yyruleno ){
  /* Beginning here are the reduction cases.  A typical example
  ** follows:
  **   case 0:
  **  #line <lineno> <grammarfile>
  **     { ... }           // User supplied code
  **  #line <lineno> <thisfile>
  **     break;
  */
      case 1: /* assignment ::= SYMBOL ASSIGN result */
#line 94 "parse.y"
{
	int i = ason_ns_mkvar(data->ns, yymsp[-2].minor.yy0.c);

	if (!i || i == -EEXIST)
		ason_ns_store(data->ns, yymsp[-2].minor.yy0.c, data->ret);
	else
		data->failed = 1;

	free(yymsp[-2].minor.yy0.c);
}
#line 814 "parse.c"
        break;
      case 2: /* result ::= equality */
#line 105 "parse.y"
{ data->ret = yymsp[0].minor.yy57; }
#line 819 "parse.c"
        break;
      case 3: /* equality ::= repr */
      case 5: /* repr ::= union */ yytestcase(yyruleno==5);
      case 7: /* union ::= intersect */ yytestcase(yyruleno==7);
      case 9: /* intersect ::= comp */ yytestcase(yyruleno==9);
      case 11: /* comp ::= join */ yytestcase(yyruleno==11);
      case 13: /* join ::= value */ yytestcase(yyruleno==13);
      case 35: /* kv_list ::= kv_pair */ yytestcase(yyruleno==35);
#line 107 "parse.y"
{ yygotominor.yy57 = yymsp[0].minor.yy57; }
#line 830 "parse.c"
        break;
      case 4: /* equality ::= equality EQUAL repr */
#line 108 "parse.y"
{
	yygotominor.yy57 = ason_equality_d(yymsp[-2].minor.yy57, yymsp[0].minor.yy57);
}
#line 837 "parse.c"
        break;
      case 6: /* repr ::= repr REPR union */
#line 113 "parse.y"
{
	yygotominor.yy57 = ason_representation_in_d(yymsp[-2].minor.yy57, yymsp[0].minor.yy57);
}
#line 844 "parse.c"
        break;
      case 8: /* union ::= union UNION intersect */
#line 119 "parse.y"
{
	yygotominor.yy57 = ason_union_d(yymsp[-2].minor.yy57, yymsp[0].minor.yy57);
}
#line 851 "parse.c"
        break;
      case 10: /* intersect ::= intersect INTERSECT comp */
#line 124 "parse.y"
{
	yygotominor.yy57 = ason_intersect_d(yymsp[-2].minor.yy57, yymsp[0].minor.yy57);
}
#line 858 "parse.c"
        break;
      case 12: /* comp ::= NOT comp */
#line 129 "parse.y"
{ yygotominor.yy57 = ason_complement_d(yymsp[0].minor.yy57); }
#line 863 "parse.c"
        break;
      case 14: /* join ::= value COLON join */
      case 36: /* kv_list ::= kv_list COMMA kv_pair */ yytestcase(yyruleno==36);
#line 132 "parse.y"
{
	yygotominor.yy57 = ason_join_d(yymsp[-2].minor.yy57, yymsp[0].minor.yy57);
}
#line 871 "parse.c"
        break;
      case 15: /* value ::= PREBAKED */
#line 136 "parse.y"
{ yygotominor.yy57 = yymsp[0].minor.yy0.value; }
#line 876 "parse.c"
        break;
      case 16: /* value ::= EMPTY */
#line 137 "parse.y"
{ yygotominor.yy57 = ASON_EMPTY; }
#line 881 "parse.c"
        break;
      case 17: /* value ::= NULL */
#line 138 "parse.y"
{ yygotominor.yy57 = ASON_NULL; }
#line 886 "parse.c"
        break;
      case 18: /* value ::= UNIVERSE */
#line 139 "parse.y"
{ yygotominor.yy57 = ASON_UNIVERSE; }
#line 891 "parse.c"
        break;
      case 19: /* value ::= WILD */
#line 140 "parse.y"
{ yygotominor.yy57 = ASON_WILD; }
#line 896 "parse.c"
        break;
      case 20: /* value ::= NUMBER */
#line 142 "parse.y"
{ yygotominor.yy57 = ason_create_fixnum(yymsp[0].minor.yy0.n); }
#line 901 "parse.c"
        break;
      case 21: /* value ::= START_LIST END_LIST */
#line 144 "parse.y"
{
	yygotominor.yy57 = ason_create_list(NULL);
}
#line 908 "parse.c"
        break;
      case 22: /* value ::= START_LIST list END_LIST */
      case 26: /* value ::= START_OBJ kv_list END_OBJ */ yytestcase(yyruleno==26);
      case 30: /* value ::= O_PAREN union C_PAREN */ yytestcase(yyruleno==30);
#line 148 "parse.y"
{ yygotominor.yy57 = yymsp[-1].minor.yy57; }
#line 915 "parse.c"
        break;
      case 23: /* value ::= TRUE */
#line 149 "parse.y"
{ yygotominor.yy57 = ASON_TRUE; }
#line 920 "parse.c"
        break;
      case 24: /* value ::= FALSE */
#line 150 "parse.y"
{ yygotominor.yy57 = ASON_FALSE; }
#line 925 "parse.c"
        break;
      case 25: /* value ::= START_OBJ END_OBJ */
#line 151 "parse.y"
{
	yygotominor.yy57 = ason_create_object(NULL,NULL);
}
#line 932 "parse.c"
        break;
      case 27: /* value ::= START_OBJ kv_list COMMA WILD END_OBJ */
#line 156 "parse.y"
{
	yygotominor.yy57 = ason_join_d(yymsp[-3].minor.yy57, ASON_OBJ_ANY);
}
#line 939 "parse.c"
        break;
      case 28: /* value ::= START_OBJ WILD END_OBJ */
#line 159 "parse.y"
{ yygotominor.yy57 = ASON_OBJ_ANY; }
#line 944 "parse.c"
        break;
      case 29: /* value ::= STRING */
#line 160 "parse.y"
{
	yygotominor.yy57 = ason_create_string(yymsp[0].minor.yy0.c);
	free(yymsp[0].minor.yy0.c);
}
#line 952 "parse.c"
        break;
      case 31: /* value ::= SYMBOL */
#line 166 "parse.y"
{
	if (data->ns)
		yygotominor.yy57 = ason_ns_load(data->ns, yymsp[0].minor.yy0.c) ?: ASON_EMPTY;
	else
		yygotominor.yy57 = ASON_EMPTY;
}
#line 962 "parse.c"
        break;
      case 32: /* list ::= union */
#line 173 "parse.y"
{ yygotominor.yy57 = ason_create_list_d(yymsp[0].minor.yy57); }
#line 967 "parse.c"
        break;
      case 33: /* list ::= union COMMA list */
#line 174 "parse.y"
{
	yygotominor.yy57 = ason_append_lists_d(ason_create_list_d(yymsp[-2].minor.yy57), yymsp[0].minor.yy57);
}
#line 974 "parse.c"
        break;
      case 34: /* kv_pair ::= STRING COLON union */
#line 178 "parse.y"
{
	yygotominor.yy57 = ason_create_object_d(yymsp[-2].minor.yy0.c, yymsp[0].minor.yy57);
	free(yymsp[-2].minor.yy0.c);
}
#line 982 "parse.c"
        break;
      default:
      /* (0) assignment ::= result */ yytestcase(yyruleno==0);
        break;
  };
  yygoto = yyRuleInfo[yyruleno].lhs;
  yysize = yyRuleInfo[yyruleno].nrhs;
  yypParser->yyidx -= yysize;
  yyact = yy_find_reduce_action(yymsp[-yysize].stateno,(YYCODETYPE)yygoto);
  if( yyact < YYNSTATE ){
#ifdef NDEBUG
    /* If we are not debugging and the reduce action popped at least
    ** one element off the stack, then we can push the new element back
    ** onto the stack here, and skip the stack overflow test in yy_shift().
    ** That gives a significant speed improvement. */
    if( yysize ){
      yypParser->yyidx++;
      yymsp -= yysize-1;
      yymsp->stateno = (YYACTIONTYPE)yyact;
      yymsp->major = (YYCODETYPE)yygoto;
      yymsp->minor = yygotominor;
    }else
#endif
    {
      yy_shift(yypParser,yyact,yygoto,&yygotominor);
    }
  }else{
    assert( yyact == YYNSTATE + YYNRULE + 1 );
    yy_accept(yypParser);
  }
}

/*
** The following code executes when the parse fails
*/
#ifndef YYNOERRORRECOVERY
static void yy_parse_failed(
  yyParser *yypParser           /* The parser */
){
  asonLemonARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sFail!\n",yyTracePrompt);
  }
#endif
  while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser fails */
  asonLemonARG_STORE; /* Suppress warning about unused %extra_argument variable */
}
#endif /* YYNOERRORRECOVERY */

/*
** The following code executes when a syntax error first occurs.
*/
static void yy_syntax_error(
  yyParser *yypParser,           /* The parser */
  int yymajor,                   /* The major type of the error token */
  YYMINORTYPE yyminor            /* The minor type of the error token */
){
  asonLemonARG_FETCH;
#define TOKEN (yyminor.yy0)
#line 57 "parse.y"
data->failed = 1;
#line 1047 "parse.c"
  asonLemonARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/*
** The following is executed when the parser accepts
*/
static void yy_accept(
  yyParser *yypParser           /* The parser */
){
  asonLemonARG_FETCH;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sAccept!\n",yyTracePrompt);
  }
#endif
  while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser accepts */
  asonLemonARG_STORE; /* Suppress warning about unused %extra_argument variable */
}

/* The main parser program.
** The first argument is a pointer to a structure obtained from
** "asonLemonAlloc" which describes the current state of the parser.
** The second argument is the major token number.  The third is
** the minor token.  The fourth optional argument is whatever the
** user wants (and specified in the grammar) and is available for
** use by the action routines.
**
** Inputs:
** <ul>
** <li> A pointer to the parser (an opaque structure.)
** <li> The major token number.
** <li> The minor token number.
** <li> An option argument of a grammar-specified type.
** </ul>
**
** Outputs:
** None.
*/
void asonLemon(
  void *yyp,                   /* The parser */
  int yymajor,                 /* The major token code number */
  asonLemonTOKENTYPE yyminor       /* The value for the token */
  asonLemonARG_PDECL               /* Optional %extra_argument parameter */
){
  YYMINORTYPE yyminorunion;
  int yyact;            /* The parser action. */
  int yyendofinput;     /* True if we are at the end of input */
#ifdef YYERRORSYMBOL
  int yyerrorhit = 0;   /* True if yymajor has invoked an error */
#endif
  yyParser *yypParser;  /* The parser */

  /* (re)initialize the parser, if necessary */
  yypParser = (yyParser*)yyp;
  if( yypParser->yyidx<0 ){
#if YYSTACKDEPTH<=0
    if( yypParser->yystksz <=0 ){
      /*memset(&yyminorunion, 0, sizeof(yyminorunion));*/
      yyminorunion = yyzerominor;
      yyStackOverflow(yypParser, &yyminorunion);
      return;
    }
#endif
    yypParser->yyidx = 0;
    yypParser->yyerrcnt = -1;
    yypParser->yystack[0].stateno = 0;
    yypParser->yystack[0].major = 0;
  }
  yyminorunion.yy0 = yyminor;
  yyendofinput = (yymajor==0);
  asonLemonARG_STORE;

#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sInput %s\n",yyTracePrompt,yyTokenName[yymajor]);
  }
#endif

  do{
    yyact = yy_find_shift_action(yypParser,(YYCODETYPE)yymajor);
    if( yyact<YYNSTATE ){
      assert( !yyendofinput );  /* Impossible to shift the $ token */
      yy_shift(yypParser,yyact,yymajor,&yyminorunion);
      yypParser->yyerrcnt--;
      yymajor = YYNOCODE;
    }else if( yyact < YYNSTATE + YYNRULE ){
      yy_reduce(yypParser,yyact-YYNSTATE);
    }else{
      assert( yyact == YY_ERROR_ACTION );
#ifdef YYERRORSYMBOL
      int yymx;
#endif
#ifndef NDEBUG
      if( yyTraceFILE ){
        fprintf(yyTraceFILE,"%sSyntax Error!\n",yyTracePrompt);
      }
#endif
#ifdef YYERRORSYMBOL
      /* A syntax error has occurred.
      ** The response to an error depends upon whether or not the
      ** grammar defines an error token "ERROR".  
      **
      ** This is what we do if the grammar does define ERROR:
      **
      **  * Call the %syntax_error function.
      **
      **  * Begin popping the stack until we enter a state where
      **    it is legal to shift the error symbol, then shift
      **    the error symbol.
      **
      **  * Set the error count to three.
      **
      **  * Begin accepting and shifting new tokens.  No new error
      **    processing will occur until three tokens have been
      **    shifted successfully.
      **
      */
      if( yypParser->yyerrcnt<0 ){
        yy_syntax_error(yypParser,yymajor,yyminorunion);
      }
      yymx = yypParser->yystack[yypParser->yyidx].major;
      if( yymx==YYERRORSYMBOL || yyerrorhit ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE,"%sDiscard input token %s\n",
             yyTracePrompt,yyTokenName[yymajor]);
        }
#endif
        yy_destructor(yypParser, (YYCODETYPE)yymajor,&yyminorunion);
        yymajor = YYNOCODE;
      }else{
         while(
          yypParser->yyidx >= 0 &&
          yymx != YYERRORSYMBOL &&
          (yyact = yy_find_reduce_action(
                        yypParser->yystack[yypParser->yyidx].stateno,
                        YYERRORSYMBOL)) >= YYNSTATE
        ){
          yy_pop_parser_stack(yypParser);
        }
        if( yypParser->yyidx < 0 || yymajor==0 ){
          yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
          yy_parse_failed(yypParser);
          yymajor = YYNOCODE;
        }else if( yymx!=YYERRORSYMBOL ){
          YYMINORTYPE u2;
          u2.YYERRSYMDT = 0;
          yy_shift(yypParser,yyact,YYERRORSYMBOL,&u2);
        }
      }
      yypParser->yyerrcnt = 3;
      yyerrorhit = 1;
#elif defined(YYNOERRORRECOVERY)
      /* If the YYNOERRORRECOVERY macro is defined, then do not attempt to
      ** do any kind of error recovery.  Instead, simply invoke the syntax
      ** error routine and continue going as if nothing had happened.
      **
      ** Applications can set this macro (for example inside %include) if
      ** they intend to abandon the parse upon the first syntax error seen.
      */
      yy_syntax_error(yypParser,yymajor,yyminorunion);
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      yymajor = YYNOCODE;
      
#else  /* YYERRORSYMBOL is not defined */
      /* This is what we do if the grammar does not define ERROR:
      **
      **  * Report an error message, and throw away the input token.
      **
      **  * If the input token is $, then fail the parse.
      **
      ** As before, subsequent error messages are suppressed until
      ** three input tokens have been successfully shifted.
      */
      if( yypParser->yyerrcnt<=0 ){
        yy_syntax_error(yypParser,yymajor,yyminorunion);
      }
      yypParser->yyerrcnt = 3;
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      if( yyendofinput ){
        yy_parse_failed(yypParser);
      }
      yymajor = YYNOCODE;
#endif
    }
  }while( yymajor!=YYNOCODE && yypParser->yyidx>=0 );
  return;
}
#line 188 "parse.y"


#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "parse.h"
#include "util.h"
#include "stringfunc.h"

/**
 * Get a number token.
 **/
static const char *
ason_get_token_number(const char *text, size_t length, int *type,
		      token_t *data)
{
	const char *text_start;
	int negator = 1;
	size_t decimal_places = 0;
	size_t decimal_inc = 0;
	size_t digits = 0;
	int64_t accum = 0;

	if (! length)
		return text;

	text_start = text;

	if (*text == '-') {
		negator = -1;
		text++;
		length--;
	}

	if (! length)
		return text_start;

	if (length < 2 && *text == 0)
		return text_start;

	if (text[0] == '0' && text[1] != '.')
		return text_start;

	for (; length; length--, text++) {
		if (*text == '.') {
			if (decimal_inc)
				return text_start;
			decimal_inc = 1;
			continue;
		} else if (! isdigit(*text)) {
			break;
		}

		digits++;
		decimal_places += decimal_inc;
		accum *= 10;
		accum += (*text) - '0';
	}

	if (decimal_inc && !decimal_places)
		return text_start;

	accum *= negator;

	/* Technically << can be undefined for negative numbers in C */
	accum = TO_FP(accum);

	while (decimal_places--)
		accum /= 10;

	if (text == text_start)
		return text_start;

	*type = ASON_LEX_NUMBER;
	data->n = accum;
	return text;
}

/**
 * Get a token from a positional argument.
 **/
static size_t
ason_get_token_arg(char type, token_t *data, int *ttype, va_list ap)
{
	size_t ret = 1;

	*ttype = ASON_LEX_PREBAKED;

	switch (type) {
	case 'i':
		data->value = ason_create_fixnum(0);
		data->value->n = TO_FP(va_arg(ap, int));
		break;
	case 'u':
		data->value = ason_create_fixnum(0);
		data->value->n = TO_FP(va_arg(ap, unsigned int));
		break;
	case 'I':
		data->value = ason_create_fixnum(0);
		data->value->n = TO_FP(va_arg(ap, int64_t));
		break;
	case 'U':
		data->value = ason_create_fixnum(0);
		data->value->n = TO_FP(va_arg(ap, uint64_t));
		break;

	/* Float becomes double in va_arg, so we can handle these together */
	case 'f':
	case 'F':
		data->value = ason_create_fixnum(0);
		/* TO_FP was designed for integers, but this seems to work */
		data->value->n = TO_FP(va_arg(ap, double));
		break;
	case 's':
		data->c = xstrdup(va_arg(ap, char *));
		*ttype = ASON_LEX_STRING;
		break;
	default:
		ret = 0;
		data->value = ason_copy(va_arg(ap, ason_t *));
	};

	return ret;
}

/**
 * Tokenize a string for ASON parsing.
 **/
static size_t
ason_get_token(const char *text, size_t length, int *type, token_t *data,
	       ason_ns_t *ns, va_list ap)
{
	const char *text_start = text;
	const char *tok_start;
	char *tmp;
	int inc;

	while (length && isspace(*text)) {
		length--;
		text++;
	}

	if (! length)
		return 0;

#define FIXED_TOKEN(s, u) do { \
	if (strlen(s) <= length && !strncmp(s, text, strlen(s))) { \
		text += strlen(s); \
		*type = u; \
		return text - text_start; \
	}  } while (0)

	FIXED_TOKEN("|", ASON_LEX_UNION);
	FIXED_TOKEN("∪", ASON_LEX_UNION);
	FIXED_TOKEN("&", ASON_LEX_INTERSECT);
	FIXED_TOKEN("∩", ASON_LEX_INTERSECT);
	FIXED_TOKEN(",", ASON_LEX_COMMA);
	FIXED_TOKEN("null", ASON_LEX_NULL);
	FIXED_TOKEN("∅", ASON_LEX_EMPTY);
	FIXED_TOKEN("_", ASON_LEX_EMPTY);
	FIXED_TOKEN("U", ASON_LEX_UNIVERSE);
	FIXED_TOKEN("*", ASON_LEX_WILD);
	FIXED_TOKEN("[", ASON_LEX_START_LIST);
	FIXED_TOKEN("]", ASON_LEX_END_LIST);
	FIXED_TOKEN("{", ASON_LEX_START_OBJ);
	FIXED_TOKEN("}", ASON_LEX_END_OBJ);
	FIXED_TOKEN(":=", ASON_LEX_ASSIGN);
	FIXED_TOKEN(":", ASON_LEX_COLON);
	FIXED_TOKEN("(", ASON_LEX_O_PAREN);
	FIXED_TOKEN(")", ASON_LEX_C_PAREN);
	FIXED_TOKEN("!", ASON_LEX_NOT);
	FIXED_TOKEN("in", ASON_LEX_REPR);
	FIXED_TOKEN("⊆", ASON_LEX_REPR);
	FIXED_TOKEN("=", ASON_LEX_EQUAL);
	FIXED_TOKEN("true", ASON_LEX_TRUE);
	FIXED_TOKEN("false", ASON_LEX_FALSE);

#undef FIXED_TOKEN

	if (*text == '?') {
		text++;
		length--;
		if (length) {
			inc = ason_get_token_arg(*text, data, type, ap);
			text += inc;
			length -= inc;
		} else {
			ason_get_token_arg('\0', data, type, ap);
		}

		return text - text_start;
	}

	tok_start = text;
	text = ason_get_token_number(text, length, type, data);

	if (text > tok_start)
		return text - text_start;

	if (*text != '"') {
		if (isdigit(*text))
			return 0;

		if (! ns)
			return 0;

		tok_start = text;

		while (length && (isalpha(*text) || isdigit(*text) ||
				  *text == '_' || *text == '.')) {
			length--;
			text++;
		}

		if (text == tok_start)
			return 0;

		data->c = xstrndup(tok_start, text - tok_start);
		*type = ASON_LEX_SYMBOL;
		return text - text_start;
	}

	tok_start = ++text;

	while (length && (*text != '"' || *(text - 1) == '\\')) {
		length--;
		text++;
	}

	if (*text != '"' || *(text - 1) == '\\')
		return 0;

	tmp = xstrndup(tok_start, text - tok_start);
	data->c = string_unescape(tmp);
	free(tmp);
	text++;
	*type = ASON_LEX_STRING;
	return text - text_start;
}

/**
 * Read an ASON value from a string. Stop after `length` bytes. Use `ns` to
 * resolve and assign symbols, and `ap` to resolve tokens.
 **/
static ason_t *
ason_ns_vreadn(const char *text, size_t length, ason_ns_t *ns, va_list ap)
{
	token_t data;
	size_t len;
	int type;
	void *parser = asonLemonAlloc(xmalloc);
	struct parse_data pdata = { .ret = NULL, .ns = ns, .failed = 0 };
	char *tmp = xstrndup(text, length);
	char *text_unicode = string_to_utf8(tmp);

	free(tmp);
	text = text_unicode;

	while ((len = ason_get_token(text, length, &type, &data, ns, ap))) {
		text += len;
		length -= len;

		asonLemon(parser, type, data, &pdata);
	}

	while (isspace(*text)) {
		text++;
		length--;
	}

	if (length)
		pdata.failed = 1;

	asonLemon(parser, 0, data, &pdata);
	asonLemonFree(parser, free);

	free(text_unicode);

	if (! pdata.failed)
		return pdata.ret;

	if (pdata.ret)
		ason_destroy(pdata.ret);

	return NULL;
}

/**
 * Read an ASON value from a string. Stop after `length` bytes. Use `ns` to
 * resolve and assign symbols.
 **/
API_EXPORT ason_t *
ason_ns_readn(ason_ns_t *ns, const char *text, size_t length, ...)
{
	va_list ap;
	ason_t *ret;

	va_start(ap, length);
	ret = ason_ns_vreadn(text, length, ns, ap);
	va_end(ap);
	return ret;
}

/**
 * Read an ASON value from a string. Use `ns` to resolve and assign symbols.
 **/
API_EXPORT ason_t *
ason_ns_read(ason_ns_t *ns, const char *text, ...)
{
	va_list ap;
	ason_t *ret;

	va_start(ap, text);
	ret = ason_ns_vreadn(text, strlen(text), ns, ap);
	va_end(ap);
	return ret;
}

/**
 * Read an ASON value from a string.
 **/
API_EXPORT ason_t *
ason_read(const char *text, ...)
{
	va_list ap;
	ason_t *ret;

	va_start(ap, text);
	ret = ason_ns_vreadn(text, strlen(text), NULL, ap);
	va_end(ap);
	return ret;
}

/**
 * Read an ASON value from a string. Stop after `length` bytes.
 **/
API_EXPORT ason_t *
ason_readn(const char *text, size_t length, ...)
{
	va_list ap;
	ason_t *ret;

	va_start(ap, length);
	ret = ason_ns_vreadn(text, length, NULL, ap);
	va_end(ap);
	return ret;
}

#line 1588 "parse.c"
